@@ -1,28 +1,41 @@
-## Copyright (C) 2006-2017 David Bateman
+########################################################################
+##
+## Copyright (C) 2006-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 function sparseimages (d, nm, typ)
+
+  if (strcmp (typ, "txt"))
+    txtimages (d, nm, 15, typ);
+    return;
+  endif
+
   set_graphics_toolkit ();
   set_print_size ();
   hide_output ();
   outfile = fullfile (d, [nm "." typ]);
   if (strcmp (typ, "png"))
-    set (0, "defaulttextfontname", "*");
+    set (groot, "defaulttextfontname", "*");
   endif
   if (strcmp (typ, "eps"))
     d_typ = "-depsc2";
@@ -36,8 +49,6 @@ function sparseimages (d, nm, typ)
     ## There is no sparse matrix implementation available because
     ## of missing libraries, plot sombreros instead.
     sombreroimage (outfile, typ, d_typ);
-  elseif (strcmp (typ, "txt"))
-    txtimages (d, nm, 15, typ);
   elseif (strcmp (nm, "gplot"))
     A = sparse ([2,6,1,3,2,4,3,5,4,6,1,5],
                 [1,1,2,2,3,3,4,4,5,5,6,6], 1, 6, 6);
@@ -244,8 +255,10 @@ endfunction
 function set_graphics_toolkit ()
   if (isempty (available_graphics_toolkits ()))
     error ("no graphics toolkit available for plotting");
-  elseif (! strcmp ("gnuplot", graphics_toolkit ())
-          && ! __have_feature__ ("OSMESA"))
+  elseif (strcmp ("qt", graphics_toolkit ())
+          && __have_feature__ ("QT_OFFSCREEN"))
+    ## Use qt with QOffscreenSurface for plot
+  elseif (! strcmp ("gnuplot", graphics_toolkit ()))
     if (! any (strcmp ("gnuplot", available_graphics_toolkits ())))
       error ("no graphics toolkit available for offscreen plotting");
     else
@@ -257,10 +270,10 @@ endfunction
 function set_print_size ()
   image_size = [5.0, 3.5]; # in inches, 16:9 format
   border = 0;              # For postscript use 50/72
-  set (0, "defaultfigurepapertype", "<custom>");
-  set (0, "defaultfigurepaperorientation", "landscape");
-  set (0, "defaultfigurepapersize", image_size + 2*border);
-  set (0, "defaultfigurepaperposition", [border, border, image_size]);
+  set (groot, "defaultfigurepapertype", "<custom>");
+  set (groot, "defaultfigurepaperorientation", "landscape");
+  set (groot, "defaultfigurepapersize", image_size + 2*border);
+  set (groot, "defaultfigurepaperposition", [border, border, image_size]);
 endfunction
 
 ## Use this function before plotting commands and after every call to print
