@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 1999-2017 John W. Eaton
-Copyright (C) 2009-2010 VZLU Prague
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1999-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -61,19 +63,20 @@ Cell::Cell (const string_vector& sv, bool trim)
     }
 }
 
-Cell::Cell (const std::list<std::string>& lst)
+Cell::Cell (const std::list<std::string>& sl)
   : Array<octave_value> ()
 {
-  size_t n = lst.size ();
+  octave_idx_type n = sl.size ();
 
   if (n > 0)
     {
       resize (dim_vector (n, 1));
 
-      octave_idx_type i = 0;
+      octave_value *dst = fortran_vec ();
+      auto p = sl.begin ();
 
-      for (const auto& str : lst)
-        elem(i++,0) = str;
+      for (octave_idx_type i = 0; i < n; i++)
+        dst[i] = *p++;
     }
 }
 
@@ -101,7 +104,7 @@ Cell::Cell (const dim_vector& dv, const string_vector& sv, bool trim)
     {
       octave_idx_type m = numel ();
 
-      octave_idx_type len = n > m ? m : n;
+      octave_idx_type len = (n > m ? m : n);
 
       for (octave_idx_type i = 0; i < len; i++)
         {
@@ -120,7 +123,7 @@ Cell::Cell (const dim_vector& dv, const string_vector& sv, bool trim)
 }
 
 bool
-Cell::is_cellstr (void) const
+Cell::iscellstr (void) const
 {
   bool retval = true;
 
@@ -151,6 +154,19 @@ Cell::cellstr_value (void) const
   return retval;
 }
 
+string_vector
+Cell::string_vector_value (void) const
+{
+  octave_idx_type n = numel ();
+
+  string_vector retval (n);
+
+  for (octave_idx_type i = 0; i < n; i++)
+    retval.xelem (i) = elem (i).string_value ();
+
+  return retval;
+}
+
 Cell
 Cell::index (const octave_value_list& idx_arg, bool resize_ok) const
 {
@@ -159,7 +175,7 @@ Cell::index (const octave_value_list& idx_arg, bool resize_ok) const
   octave_idx_type n = idx_arg.length ();
 
   // If we catch an indexing error in index_vector, we flag an error
-  // in index k.  Ensure it is the right value befor each idx_vector
+  // in index k.  Ensure it is the right value before each idx_vector
   // call.  Same variable as used in for loop in default case.
 
   octave_idx_type k = 0;
@@ -169,6 +185,7 @@ Cell::index (const octave_value_list& idx_arg, bool resize_ok) const
       switch (n)
         {
         case 0:
+          warn_empty_index ("cell array");
           retval = *this;
           break;
 
@@ -214,9 +231,11 @@ Cell::index (const octave_value_list& idx_arg, bool resize_ok) const
 }
 
 /*
-%!test
+%% This behavior is required for Matlab compatibility.
+%!shared a
 %! a = {"foo", "bar"};
-%! assert (a(), a);
+%!assert (a(), a);
+%!error <invalid empty index expression> a{}
 */
 
 void
