@@ -1,28 +1,30 @@
-/*
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1998-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
-Copyright (C) 2004-2013 David Bateman
-Copyright (C) 1998-2004 Andy Adler
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
+#if defined (HAVE_CONFIG_H)
+#  include "config.h"
 #endif
 
 #include <cassert>
@@ -34,7 +36,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "quit.h"
 
 #include "error.h"
-#include "oct-obj.h"
+#include "ovl.h"
 #include "utils.h"
 
 #include "dSparse.h"
@@ -46,13 +48,12 @@ along with Octave; see the file COPYING.  If not, see
 static inline int
 xisint (double x)
 {
-  return (D_NINT (x) == x
+  return (octave::math::x_nint (x) == x
           && ((x >= 0 && x < std::numeric_limits<int>::max ())
               || (x <= 0 && x > std::numeric_limits<int>::min ())));
 }
 
-
-// Safer pow functions. Only two make sense for sparse matrices, the
+// Safer pow functions.  Only two make sense for sparse matrices, the
 // others should all promote to full matrices.
 
 octave_value
@@ -64,65 +65,60 @@ xpow (const SparseMatrix& a, double b)
   octave_idx_type nc = a.cols ();
 
   if (nr == 0 || nc == 0 || nr != nc)
-    error ("for A^b, A must be a square matrix. Use .^ for elementwise power.");
+    error ("for A^b, A must be a square matrix.  Use .^ for elementwise power.");
+
+  if (! xisint (b))
+    error ("use full(a) ^ full(b)");
+
+  int btmp = static_cast<int> (b);
+  if (btmp == 0)
+    {
+      SparseMatrix tmp = SparseMatrix (nr, nr, nr);
+      for (octave_idx_type i = 0; i < nr; i++)
+        {
+          tmp.data (i) = 1.0;
+          tmp.ridx (i) = i;
+        }
+      for (octave_idx_type i = 0; i < nr + 1; i++)
+        tmp.cidx (i) = i;
+
+      retval = tmp;
+    }
   else
     {
-      if (static_cast<int> (b) == b)
+      SparseMatrix atmp;
+      if (btmp < 0)
         {
-          int btmp = static_cast<int> (b);
-          if (btmp == 0)
-            {
-              SparseMatrix tmp = SparseMatrix (nr, nr, nr);
-              for (octave_idx_type i = 0; i < nr; i++)
-                {
-                  tmp.data (i) = 1.0;
-                  tmp.ridx (i) = i;
-                }
-              for (octave_idx_type i = 0; i < nr + 1; i++)
-                tmp.cidx (i) = i;
+          btmp = -btmp;
 
-              retval = tmp;
-            }
-          else
-            {
-              SparseMatrix atmp;
-              if (btmp < 0)
-                {
-                  btmp = -btmp;
+          octave_idx_type info;
+          double rcond = 0.0;
+          MatrixType mattyp (a);
 
-                  octave_idx_type info;
-                  double rcond = 0.0;
-                  MatrixType mattyp (a);
+          atmp = a.inverse (mattyp, info, rcond, 1);
 
-                  atmp = a.inverse (mattyp, info, rcond, 1);
-
-                  if (info == -1)
-                    warning ("inverse: matrix singular to machine\
- precision, rcond = %g", rcond);
-                }
-              else
-                atmp = a;
-
-              SparseMatrix result (atmp);
-
-              btmp--;
-
-              while (btmp > 0)
-                {
-                  if (btmp & 1)
-                    result = result * atmp;
-
-                  btmp >>= 1;
-
-                  if (btmp > 0)
-                    atmp = atmp * atmp;
-                }
-
-              retval = result;
-            }
+          if (info == -1)
+            warning ("inverse: matrix singular to machine precision, rcond = %g", rcond);
         }
       else
-        error ("use full(a) ^ full(b)");
+        atmp = a;
+
+      SparseMatrix result (atmp);
+
+      btmp--;
+
+      while (btmp > 0)
+        {
+          if (btmp & 1)
+            result = result * atmp;
+
+          btmp >>= 1;
+
+          if (btmp > 0)
+            atmp = atmp * atmp;
+        }
+
+      retval = result;
     }
 
   return retval;
@@ -137,65 +133,60 @@ xpow (const SparseComplexMatrix& a, double b)
   octave_idx_type nc = a.cols ();
 
   if (nr == 0 || nc == 0 || nr != nc)
-    error ("for A^b, A must be a square matrix. Use .^ for elementwise power.");
+    error ("for A^b, A must be a square matrix.  Use .^ for elementwise power.");
+
+  if (! xisint (b))
+    error ("use full(a) ^ full(b)");
+
+  int btmp = static_cast<int> (b);
+  if (btmp == 0)
+    {
+      SparseMatrix tmp = SparseMatrix (nr, nr, nr);
+      for (octave_idx_type i = 0; i < nr; i++)
+        {
+          tmp.data (i) = 1.0;
+          tmp.ridx (i) = i;
+        }
+      for (octave_idx_type i = 0; i < nr + 1; i++)
+        tmp.cidx (i) = i;
+
+      retval = tmp;
+    }
   else
     {
-      if (static_cast<int> (b) == b)
+      SparseComplexMatrix atmp;
+      if (btmp < 0)
         {
-          int btmp = static_cast<int> (b);
-          if (btmp == 0)
-            {
-              SparseMatrix tmp = SparseMatrix (nr, nr, nr);
-              for (octave_idx_type i = 0; i < nr; i++)
-                {
-                  tmp.data (i) = 1.0;
-                  tmp.ridx (i) = i;
-                }
-              for (octave_idx_type i = 0; i < nr + 1; i++)
-                tmp.cidx (i) = i;
+          btmp = -btmp;
 
-              retval = tmp;
-            }
-          else
-            {
-              SparseComplexMatrix atmp;
-              if (btmp < 0)
-                {
-                  btmp = -btmp;
+          octave_idx_type info;
+          double rcond = 0.0;
+          MatrixType mattyp (a);
 
-                  octave_idx_type info;
-                  double rcond = 0.0;
-                  MatrixType mattyp (a);
+          atmp = a.inverse (mattyp, info, rcond, 1);
 
-                  atmp = a.inverse (mattyp, info, rcond, 1);
-
-                  if (info == -1)
-                    warning ("inverse: matrix singular to machine\
- precision, rcond = %g", rcond);
-                }
-              else
-                atmp = a;
-
-              SparseComplexMatrix result (atmp);
-
-              btmp--;
-
-              while (btmp > 0)
-                {
-                  if (btmp & 1)
-                    result = result * atmp;
-
-                  btmp >>= 1;
-
-                  if (btmp > 0)
-                    atmp = atmp * atmp;
-                }
-
-              retval = result;
-            }
+          if (info == -1)
+            warning ("inverse: matrix singular to machine precision, rcond = %g", rcond);
         }
       else
-        error ("use full(a) ^ full(b)");
+        atmp = a;
+
+      SparseComplexMatrix result (atmp);
+
+      btmp--;
+
+      while (btmp > 0)
+        {
+          if (btmp & 1)
+            result = result * atmp;
+
+          btmp >>= 1;
+
+          if (btmp > 0)
+            atmp = atmp * atmp;
+        }
+
+      retval = result;
     }
 
   return retval;
@@ -236,21 +227,21 @@ xpow (const SparseComplexMatrix& a, double b)
 // seem worth the effort to optimize -- how often does this case come up
 // in practice?
 
-template <class S, class SM>
+template <typename S, typename SM>
 inline octave_value
 scalar_xpow (const S& a, const SM& b)
 {
   octave_value val = elem_xpow (a, b);
 
-  if (val.is_complex_type ())
+  if (val.iscomplex ())
     return SparseComplexMatrix (val.complex_matrix_value ());
   else
     return SparseMatrix (val.matrix_value ());
 }
 
 /*
-%!assert (sparse (2) .^ [3, 4], sparse ([8, 16]));
-%!assert (sparse (2i) .^ [3, 4], sparse ([-0-8i, 16]));
+%!assert (sparse (2) .^ [3, 4], sparse ([8, 16]))
+%!assert <47775> (sparse (2i) .^ [3, 4], sparse ([-0-8i, 16]))
 */
 
 // -*- 1 -*-
@@ -327,7 +318,7 @@ elem_xpow (const SparseMatrix& a, double b)
 {
   // FIXME: What should a .^ 0 give?  Matlab gives a
   // sparse matrix with same structure as a, which is strictly
-  // incorrect. Keep compatibility.
+  // incorrect.  Keep compatibility.
 
   octave_value retval;
 
@@ -338,7 +329,7 @@ elem_xpow (const SparseMatrix& a, double b)
       octave_idx_type nr = a.rows ();
       octave_idx_type nc = a.cols ();
 
-      if (static_cast<int> (b) != b && a.any_element_is_negative ())
+      if (! xisint (b) && a.any_element_is_negative ())
         {
           ComplexMatrix result (nr, nc, Complex (std::pow (0.0, b)));
 
@@ -372,7 +363,7 @@ elem_xpow (const SparseMatrix& a, double b)
           retval = octave_value (result);
         }
     }
-  else if (static_cast<int> (b) != b && a.any_element_is_negative ())
+  else if (! xisint (b) && a.any_element_is_negative ())
     {
       SparseComplexMatrix result (a);
 
@@ -427,10 +418,7 @@ elem_xpow (const SparseMatrix& a, const SparseMatrix& b)
     return scalar_xpow (a(0), b);
 
   if (nr != b_nr || nc != b_nc)
-    {
-      gripe_nonconformant ("operator .^", nr, nc, b_nr, b_nc);
-      return octave_value ();
-    }
+    octave::err_nonconformant ("operator .^", nr, nc, b_nr, b_nc);
 
   int convert_to_complex = 0;
   for (octave_idx_type j = 0; j < nc; j++)
@@ -439,7 +427,7 @@ elem_xpow (const SparseMatrix& a, const SparseMatrix& b)
         if (a.data(i) < 0.0)
           {
             double btmp = b (a.ridx (i), j);
-            if (static_cast<int> (btmp) != btmp)
+            if (! xisint (btmp))
               {
                 convert_to_complex = 1;
                 goto done;
@@ -450,9 +438,9 @@ elem_xpow (const SparseMatrix& a, const SparseMatrix& b)
 done:
 
   // This is a dumb operator for sparse matrices anyway, and there is
-  // no sensible way to handle the 0.^0 versus the 0.^x cases. Therefore
+  // no sensible way to handle the 0.^0 versus the 0.^x cases.  Therefore
   // allocate a full matrix filled for the 0.^0 case and shrink it later
-  // as needed
+  // as needed.
 
   if (convert_to_complex)
     {
@@ -463,8 +451,8 @@ done:
           for (octave_idx_type i = a.cidx (j); i < a.cidx (j+1); i++)
             {
               octave_quit ();
-              complex_result.xelem (a.ridx (i), j) =
-                std::pow (Complex (a.data (i)), Complex (b(a.ridx (i), j)));
+              complex_result.xelem (a.ridx (i), j)
+                = std::pow (Complex (a.data (i)), Complex (b(a.ridx (i), j)));
             }
         }
       complex_result.maybe_compress (true);
@@ -532,10 +520,7 @@ elem_xpow (const SparseMatrix& a, const SparseComplexMatrix& b)
     return scalar_xpow (a(0), b);
 
   if (nr != b_nr || nc != b_nc)
-    {
-      gripe_nonconformant ("operator .^", nr, nc, b_nr, b_nc);
-      return octave_value ();
-    }
+    octave::err_nonconformant ("operator .^", nr, nc, b_nr, b_nc);
 
   SparseComplexMatrix result (nr, nc, Complex (1.0, 0.0));
   for (octave_idx_type j = 0; j < nc; j++)
@@ -614,8 +599,8 @@ elem_xpow (const SparseComplexMatrix& a, double b)
             for (octave_idx_type i = a.cidx (j); i < a.cidx (j+1); i++)
               {
                 octave_quit ();
-                result (a.ridx (i), j) =
-                  std::pow (a.data (i), static_cast<int> (b));
+                result (a.ridx (i), j)
+                  = std::pow (a.data (i), static_cast<int> (b));
               }
         }
       else
@@ -675,10 +660,7 @@ elem_xpow (const SparseComplexMatrix& a, const SparseMatrix& b)
     return scalar_xpow (a(0), b);
 
   if (nr != b_nr || nc != b_nc)
-    {
-      gripe_nonconformant ("operator .^", nr, nc, b_nr, b_nc);
-      return octave_value ();
-    }
+    octave::err_nonconformant ("operator .^", nr, nc, b_nr, b_nc);
 
   SparseComplexMatrix result (nr, nc, Complex (1.0, 0.0));
   for (octave_idx_type j = 0; j < nc; j++)
@@ -687,11 +669,10 @@ elem_xpow (const SparseComplexMatrix& a, const SparseMatrix& b)
         {
           octave_quit ();
           double btmp = b(a.ridx (i), j);
-          Complex tmp;
 
           if (xisint (btmp))
             result.xelem (a.ridx (i), j) = std::pow (a.data (i),
-                                              static_cast<int> (btmp));
+                                                     static_cast<int> (btmp));
           else
             result.xelem (a.ridx (i), j) = std::pow (a.data (i), btmp);
         }
@@ -746,10 +727,7 @@ elem_xpow (const SparseComplexMatrix& a, const SparseComplexMatrix& b)
     return scalar_xpow (a(0), b);
 
   if (nr != b_nr || nc != b_nc)
-    {
-      gripe_nonconformant ("operator .^", nr, nc, b_nr, b_nc);
-      return octave_value ();
-    }
+    octave::err_nonconformant ("operator .^", nr, nc, b_nr, b_nc);
 
   SparseComplexMatrix result (nr, nc, Complex (1.0, 0.0));
   for (octave_idx_type j = 0; j < nc; j++)
