@@ -1,30 +1,33 @@
-/*
-
-Copyright (C) 2005-2017 David Bateman
-Copyright (C) 1998-2005 Andy Adler
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1998-2021 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
 #endif
 
+#include <cinttypes>
 #include <cstdarg>
 #include <cstdio>
 
@@ -101,8 +104,9 @@ sparse_indices_ok (octave_idx_type *r, octave_idx_type *c,
 
           if (c[j] > nnz)
             (*current_liboctave_error_handler)
-              ("invalid sparse matrix: cidx[%d] = %d "
-               "exceeds number of nonzero elements", j, c[j]+1);
+              ("invalid sparse matrix: cidx[%" OCTAVE_IDX_TYPE_FORMAT "] = "
+               "%" OCTAVE_IDX_TYPE_FORMAT " exceeds number of nonzero elements",
+               j, c[j]+1);
 
           if (c[j] != jold)
             {
@@ -115,7 +119,8 @@ sparse_indices_ok (octave_idx_type *r, octave_idx_type *c,
 
                   if (r[i] >= nrows)
                     (*current_liboctave_error_handler)
-                      ("invalid sparse matrix: ridx[%d] = %d out of range",
+                      ("invalid sparse matrix: ridx[%" OCTAVE_IDX_TYPE_FORMAT
+                       "] = %" OCTAVE_IDX_TYPE_FORMAT " out of range",
                        i, r[i]+1);
                 }
 
