@@ -1,20 +1,27 @@
-## Copyright (C) 1993-2017 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1993-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} logspace (@var{a}, @var{b})
@@ -65,25 +72,23 @@
 ## @seealso{linspace}
 ## @end deftypefn
 
-## Author: jwe
-
-function retval = logspace (base, limit, n = 50)
+function retval = logspace (a, b, n = 50)
 
   if (nargin != 2 && nargin != 3)
     print_usage ();
   endif
 
-  if (! (isscalar (base) && isscalar (limit) && isscalar (n)))
-    error ("logspace: arguments BASE, LIMIT, and N must be scalars");
+  if (! (isscalar (a) && isscalar (b) && isscalar (n)))
+    error ("logspace: arguments A, B, and N must be scalars");
   endif
 
   npoints = fix (n);
 
-  if (limit == pi)
-    limit = log10 (pi);
+  if (b == pi)
+    b = log10 (pi);
   endif
 
-  retval = 10 .^ (linspace (base, limit, npoints));
+  retval = 10 .^ (linspace (a, b, npoints));
 
 endfunction
 
@@ -98,6 +103,21 @@ endfunction
 %! assert (size (x3) == [1, 10] && abs (x3(1) - 10) < eps && abs (x3(10) - 0.01) < eps);
 %! assert (size (x4) == [1, 10] && abs (x4(1) - 10) < eps && abs (x4(10) - pi) < sqrt (eps));
 
+## Edge cases
+%!assert (logspace (Inf, Inf, 3), [Inf, Inf, Inf])
+%!assert (logspace (-Inf, Inf, 3), [0, 1, Inf])
+%!assert (logspace (Inf + 1i, Inf + 1i, 3), repmat (complex (-Inf,Inf), [1, 3]))
+%!assert (logspace (-Inf + 1i, Inf + 1i, 3), [0, NaN + NaN * 1i, complex(-Inf, Inf)])
+%!assert (logspace (0, Inf, 3), [1, Inf, Inf])
+%!assert (logspace (0, -Inf, 3), [1, 0, 0])
+%!assert (logspace (Inf, -Inf, 3), [Inf, 1, 0])
+
+## FIXME: These are bizarre corner cases for Matlab compatibility.  See
+## bug #56933.  This is marked as "Won't Fix", but if linspace is updated at
+## some point then these tests can be re-instated.
+##%!assert (logspace (-Inf, 0, 3), [0, NaN, 1])
+##%!assert (logspace (Inf, 0, 3), [Inf, NaN, 1])
+
 ## Test input validation
 %!error logspace ()
 %!error logspace (1, 2, 3, 4)
