@@ -1,4 +1,9 @@
-## Copyright (C) 1993-2018 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1993-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} xlabel (@var{string})
@@ -26,6 +33,9 @@
 ## An optional list of @var{property}/@var{value} pairs can be used to change
 ## the properties of the created text label.
 ##
+## The full list of text object properties is documented at
+## @ref{Text Properties}.
+##
 ## If the first argument @var{hax} is an axes handle, then operate on
 ## this axes rather than the current axes returned by @code{gca}.
 ##
@@ -34,8 +44,6 @@
 ## @seealso{ylabel, zlabel, datetick, title, text}
 ## @end deftypefn
 
-## Author: jwe
-
 function h = xlabel (varargin)
 
   [hax, varargin, nargin] = __plt_get_axis_arg__ ("xlabel", varargin{:});
