@@ -1,20 +1,27 @@
-## Copyright (C) 1994-2017 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1994-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn {} {} roots (@var{c})
@@ -75,39 +82,35 @@
 ## @seealso{poly, compan, fzero}
 ## @end deftypefn
 
-## Author: KH <Kurt.Hornik@wu-wien.ac.at>
-## Created: 24 December 1993
-## Adapted-By: jwe
+function r = roots (c)
 
-function r = roots (v)
-
-  if (nargin != 1 || (! isvector (v) && ! isempty (v)))
+  if (nargin != 1 || (! isvector (c) && ! isempty (c)))
     print_usage ();
-  elseif (any (! isfinite (v)))
+  elseif (any (! isfinite (c)))
     error ("roots: inputs must not contain Inf or NaN");
   endif
 
-  v = v(:);
-  n = numel (v);
+  c = c(:);
+  n = numel (c);
 
-  ## If v = [ 0 ... 0 v(k+1) ... v(k+l) 0 ... 0 ],
+  ## If c = [ 0 ... 0 c(k+1) ... c(k+l) 0 ... 0 ],
   ## we can remove the leading k zeros,
   ## and n - k - l roots of the polynomial are zero.
 
-  v_max = max (abs (v));
-  if (isempty (v) || v_max == 0)
+  c_max = max (abs (c));
+  if (isempty (c) || c_max == 0)
     r = [];
     return;
   endif
 
-  f = find (v ./ v_max);
+  f = find (c ./ c_max);
   m = numel (f);
 
-  v = v(f(1):f(m));
-  l = numel (v);
+  c = c(f(1):f(m));
+  l = numel (c);
   if (l > 1)
     A = diag (ones (1, l-2), -1);
-    A(1,:) = -v(2:l) ./ v(1);
+    A(1,:) = -c(2:l) ./ c(1);
     r = eig (A);
     if (f(m) < n)
       r = [r; zeros(n - f(m), 1)];
