@@ -1,23 +1,30 @@
-## Copyright (C) 1994-2015 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1994-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} sinc (@var{x})
+## @deftypefn {} {} sinc (@var{x})
 ## Compute the sinc function.
 ##
 ## Return
@@ -29,8 +36,6 @@
 ## @end ifnottex
 ## @end deftypefn
 
-## Author: jwe ???
-
 function result = sinc (x)
 
   if (nargin != 1)
@@ -54,4 +59,3 @@ endfunction
 %!assert (sinc (1/2), 2/pi, 1e-6)
 
 %!error sinc()
-
