@@ -1,25 +1,33 @@
-## Copyright (C) 1993-2012 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1993-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} hilb (@var{n})
-## Return the Hilbert matrix of order @var{n}.  The @math{i,j} element
-## of a Hilbert matrix is defined as
+## @deftypefn {} {} hilb (@var{n})
+## Return the Hilbert matrix of order @var{n}.
+##
+## The @math{i,j} element of a Hilbert matrix is defined as
 ## @tex
 ## $$
 ## H(i, j) = {1 \over (i + j - 1)}
@@ -34,9 +42,9 @@
 ## @end ifnottex
 ##
 ## Hilbert matrices are close to being singular which make them difficult to
-## invert with numerical routines.
-## Comparing the condition number of a random matrix 5x5 matrix with that of
-## a Hilbert matrix of order 5 reveals just how difficult the problem is.
+## invert with numerical routines.  Comparing the condition number of a random
+## matrix 5x5 matrix with that of a Hilbert matrix of order 5 reveals just how
+## difficult the problem is.
 ##
 ## @example
 ## @group
@@ -50,8 +58,6 @@
 ## @seealso{invhilb}
 ## @end deftypefn
 
-## Author: jwe
-
 function retval = hilb (n)
 
   if (nargin != 1)
@@ -60,12 +66,10 @@ function retval = hilb (n)
     error ("hilb: N must be a scalar integer");
   endif
 
-  retval = zeros (n);
-  tmp = 1:n;
-  for i = 1:n
-    retval(i, :) = 1.0 ./ tmp;
-    tmp++;
-  endfor
+  ## Very elegant solution by N. Higham
+  ## https://nhigham.com/2020/06/30/what-is-the-hilbert-matrix/
+  j = 1:n;
+  retval = 1 ./ (j' + j - 1);
 
 endfunction
 
@@ -76,4 +80,3 @@ endfunction
 %!error hilb ()
 %!error hilb (1, 2)
 %!error <N must be a scalar integer> hilb (ones (2))
-
