@@ -1,30 +1,37 @@
-## Copyright (C) 1994-2012 John W. Eaton
-## Copyright (C) 2009 Jaroslav Hajek
+########################################################################
+##
+## Copyright (C) 1994-2021 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} strcat (@var{s1}, @var{s2}, @dots{})
+## @deftypefn {} {} strcat (@var{s1}, @var{s2}, @dots{})
 ## Return a string containing all the arguments concatenated
-## horizontally.  If the arguments are cell strings, @code{strcat}
-## returns a cell string with the individual cells concatenated.
-## For numerical input, each element is converted to the
-## corresponding ASCII character.  Trailing white space for any
-## character string input is eliminated before the strings are
+## horizontally.
+##
+## If the arguments are cell strings, @code{strcat} returns a cell string
+## with the individual cells concatenated.  For numerical input, each element
+## is converted to the corresponding ASCII character.  Trailing white space
+## for any character string input is eliminated before the strings are
 ## concatenated.  Note that cell string values do @strong{not} have
 ## whitespace trimmed.
 ##
@@ -56,7 +63,7 @@
 ## s = [ "ab"; "cde" ];
 ## strcat (s, s, s)
 ##     @result{}
-##         "ab ab ab "
+##         "ababab   "
 ##         "cdecdecde"
 ## @end group
 ## @end example
@@ -68,7 +75,7 @@
 ##     @result{}
 ##         @{
 ##           [1,1] = ababab
-##           [2,1] = cd cd cd 
+##           [2,1] = cd cd cd
 ##         @}
 ## @end group
 ## @end example
@@ -76,16 +83,18 @@
 ## @seealso{cstrcat, char, strvcat}
 ## @end deftypefn
 
-## Author: jwe
-
 function st = strcat (varargin)
 
   if (nargin == 0)
-    print_usage ();
-  endif
-
-  if (nargin == 1)
-    st = varargin{1};
+    st = "";
+  elseif (nargin == 1)
+    if (iscellstr (varargin{1}))
+      st = varargin{1};
+    elseif (isreal (varargin{1}) || ischar (varargin{1}))
+      st = char (cellstr (varargin{1}));
+    else
+      error ("strcat: inputs must be strings or cells of strings");
+    endif
   else
     ## Convert to cells of strings
     uo = "uniformoutput";
@@ -133,7 +142,7 @@ endfunction
 
 ## test for deblanking implied trailing spaces of character input
 %!assert (strcat ("foo", "bar"), "foobar")
-%!assert (strcat (["a"; "bb"], ["foo"; "bar"]), ["afoo "; "bbbar"])
+%!assert (strcat (["a "; "bb"], ["foo"; "bar"]), ["afoo "; "bbbar"])
 
 ## test for mixing character and cell inputs
 %!assert (strcat ("a", {"bc", "de"}, "f"), {"abcf", "adef"})
@@ -145,8 +154,12 @@ endfunction
 %!assert (all (strcmp (strcat ({"a", "bb"}, "ccc"), {"accc", "bbccc"})))
 %!assert (all (strcmp (strcat ("a", {"bb", "ccc"}), {"abb", "accc"})))
 
+## test with a single string or cell input
+%!assert <*49094> (strcat ("foo    "), "foo")
+%!assert <*49094> (strcat ({"foo"}), {"foo"})
+
+%!assert (strcat (1), char (1))
 %!assert (strcat (1, 2), strcat (char (1), char (2)))
 %!assert (strcat ("", 2), strcat ([], char (2)))
 
-%!error strcat ()
-
+%!assert (strcat (), "")
